<?php
/**
 * agreement.php
 *
 * User Agreement plugin for MyBB 1.8
 * Copyright (c) 2018 doylecc. All rights reserved.
 * Website: http://mybbplugins.tk
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

// Disallow direct access to this file for security reasons
if (!defined("IN_MYBB")) {
    die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}

// Add the front end hooks
$plugins->add_hook("global_start", "agreement_lang");
$plugins->add_hook("global_end", "agreement_force");
$plugins->add_hook("member_profile_start", "agreement_save");
$plugins->add_hook("member_profile_start", "agreement_show");
$plugins->add_hook("datahandler_user_insert", "agreement_date");
$plugins->add_hook("usercp_start", "agreement_usercp");
$plugins->add_hook("member_profile_end", "agreement_profile");

// Plugin info
function agreement_info()
{
    global $mybb, $lang;

    $lang->load('agreement');

    $agreement_info = array(
        "name"          => $lang->ag_name,
        "description"   => $lang->ag_name_descr,
        "website"       => "http://mybbplugins.tk",
        "author"        => "doylecc",
        "authorsite"    => "http://mybbplugins.tk",
        "version"       => "1.3",
        "codename"      => "dcc_agreement",
        "compatibility" => "18*"
    );

    $agreement_info['description'] .= '<ul><li style="list-style: none;"><img src="styles/default/images/icons/';
    $agreement_info['description'] .= 'run_task.png" alt="" style="margin-left: 10px;" /><a href="index.php?module=';
    $agreement_info['description'] .= 'tools-agreement&amp;action=reset&amp;my_post_key=';
    $agreement_info['description'] .= $mybb->post_code.'" style="margin: 10px;">';
    $agreement_info['description'] .= $lang->ag_reset.'</a></li></ul>';

    return $agreement_info;
}

// Install function
function agreement_install()
{
    global $db, $lang;

    // Load language file
    $lang->load('agreement');

    // Avoid database errors due to duplicates
    if ($db->field_exists("new_terms", "users")) {
        $db->write_query("ALTER TABLE `".TABLE_PREFIX."users` DROP COLUMN `new_terms`");
    }
    if ($db->field_exists("new_terms_date", "users")) {
        $db->write_query("ALTER TABLE `".TABLE_PREFIX."users` DROP COLUMN `new_terms_date`");
    }
    // Add database columns
    switch ($db->type) {
        case "pgsql":
            $db->write_query(
                "ALTER TABLE `".TABLE_PREFIX."users` ADD COLUMN `new_terms` smallint NOT NULL DEFAULT '0'"
            );
            $db->write_query(
                "ALTER TABLE `".TABLE_PREFIX."users` ADD COLUMN `new_terms_date` int NOT NULL default '0'"
            );
            break;
        default:
            $db->write_query(
                "ALTER TABLE `".TABLE_PREFIX."users` ADD COLUMN `new_terms` tinyint(1) NOT NULL DEFAULT '0'"
            );
            $db->write_query(
                "ALTER TABLE `".TABLE_PREFIX."users` ADD COLUMN `new_terms_date` int unsigned NOT NULL default '0'"
            );
            break;
    }

    // Delete former settings to avoid duplicates
    $query = $db->simple_select('settinggroups', 'gid', 'name="UserAgreement"');
    $ag = $db->fetch_array($query);
    $db->delete_query('settinggroups', "gid='".$ag['gid']."'");
    $db->delete_query('settings', "gid='".$ag['gid']."'");

    // Add settings
    $query_rows = $db->simple_select("settinggroups", "COUNT(*) as rows");
    $rows = $db->fetch_field($query_rows, "rows");

    // Add Settinggroup
    $agreement_group = array(
        "name" => "UserAgreement",
        "title" => $db->escape_string($lang->ag_group_title),
        "description" => $db->escape_string($lang->ag_group_descr),
        "disporder" => $rows+1,
        "isdefault" => 0
    );
    $db->insert_query("settinggroups", $agreement_group);
    $gid = $db->insert_id();

    // Add settings for settinggroup
    $agreement_1 = array(
        "name" => "ag_force",
        "title" => $db->escape_string($lang->ag_force_title),
        "description" => $db->escape_string($lang->ag_force_descr),
        "optionscode" => "yesno",
        "value" => 1,
        "disporder" => 1,
        "gid" => (int)$gid
        );
    $db->insert_query("settings", $agreement_1);

    $agreement_2 = array(
        "name" => "ag_repeat",
        "title" => $db->escape_string($lang->ag_repeat_title),
        "description" => $db->escape_string($lang->ag_repeat_descr),
        "optionscode" => "yesno",
        "value" => 0,
        "disporder" => 2,
        "gid" => (int)$gid
        );
    $db->insert_query("settings", $agreement_2);

    $agreement_3 = array(
        "name" => "ag_allowedpages",
        "title" => $db->escape_string($lang->ag_allowedpages_title),
        "description" => $db->escape_string($lang->ag_allowedpages_descr),
        "optionscode" => "textarea",
        "value" => '',
        "disporder" => 3,
        "gid" => (int)$gid
        );
    $db->insert_query("settings", $agreement_3);

    $agreement_4 = array(
        "name" => "ag_usercp",
        "title" => $db->escape_string($lang->ag_usercp_title),
        "description" => $db->escape_string($lang->ag_usercp_descr),
        "optionscode" => "yesno",
        "value" => 1,
        "disporder" => 4,
        "gid" => (int)$gid
        );
    $db->insert_query("settings", $agreement_4);

    $agreement_5 = array(
        "name" => "ag_profile",
        "title" => $db->escape_string($lang->ag_profile_title),
        "description" => $db->escape_string($lang->ag_profile_descr),
        "optionscode" => "yesno",
        "value" => 1,
        "disporder" => 5,
        "gid" => (int)$gid
        );
    $db->insert_query("settings", $agreement_5);

    $agreement_6 = array(
        "name" => "ag_lang",
        "title" => $db->escape_string($lang->ag_lang_title),
        "description" => $db->escape_string($lang->ag_lang_descr),
        "optionscode" => "yesno",
        "value" => 0,
        "disporder" => 6,
        "gid" => (int)$gid
        );
    $db->insert_query("settings", $agreement_6);

    $agreement_7 = array(
        "name" => "ag_langtitle",
        "title" => $db->escape_string($lang->ag_langtitle_title),
        "description" => $db->escape_string($lang->ag_langtitle_descr),
        "optionscode" => "text",
        "value" => '',
        "disporder" => 7,
        "gid" => (int)$gid
        );
    $db->insert_query("settings", $agreement_7);

    $agreement_8 = array(
        "name" => "ag_ipaddresses",
        "title" => $db->escape_string($lang->ag_ipaddresses_title),
        "description" => $db->escape_string($lang->ag_ipaddresses_descr),
        "optionscode" => "select\n
never=".$db->escape_string($lang->ag_never)."\n
day=".$db->escape_string($lang->ag_day)."\n
threedays=".$db->escape_string($lang->ag_threedays)."\n
week=".$db->escape_string($lang->ag_week)."\n
twoweeks=".$db->escape_string($lang->ag_twoweeks)."\n
month=".$db->escape_string($lang->ag_month)."\n
tenweeks=".$db->escape_string($lang->ag_tenweeks)."\n
halfyear=".$db->escape_string($lang->ag_halfyear)."\n
year=".$db->escape_string($lang->ag_year)."\n
forever=".$db->escape_string($lang->ag_forever)."\n",
        "value" => "forever",
        "disporder" => 8,
        "gid" => (int)$gid
        );
    $db->insert_query("settings", $agreement_8);

    $agreement_9 = array(
        "name" => "ag_ip_del_user",
        "title" => $db->escape_string($lang->ag_ip_del_user_title),
        "description" => $db->escape_string($lang->ag_ip_del_user_descr),
        "optionscode" => "yesno",
        "value" => 1,
        "disporder" => 9,
        "gid" => (int)$gid
        );
    $db->insert_query("settings", $agreement_9);

    $agreement_10 = array(
        "name" => "ag_guest_post",
        "title" => $db->escape_string($lang->ag_guest_post_title),
        "description" => $db->escape_string($lang->ag_guest_post_descr),
        "optionscode" => "yesno",
        "value" => 1,
        "disporder" => 10,
        "gid" => (int)$gid
        );
    $db->insert_query("settings", $agreement_10);

    // Refresh settings.php
    rebuild_settings();
}

// Check if plugin is installed
function agreement_is_installed()
{
    global $db;

    if ($db->field_exists("new_terms", "users") && $db->field_exists("new_terms_date", "users")) {
        return true;
    } else {
        return false;
    }
}

// The uninstall function
function agreement_uninstall()
{
    global $db;

    // Delete database columns
    if ($db->field_exists("new_terms", "users")) {
        $db->write_query("ALTER TABLE `".TABLE_PREFIX."users` DROP COLUMN `new_terms`");
    }
    if ($db->field_exists("new_terms_date", "users")) {
        $db->write_query("ALTER TABLE `".TABLE_PREFIX."users` DROP COLUMN `new_terms_date`");
    }

    // Delete the settings
    $query = $db->simple_select('settinggroups', 'gid', 'name="UserAgreement"');
    $ag = $db->fetch_array($query);
    $db->delete_query('settinggroups', "gid='".$ag['gid']."'");
    $db->delete_query('settings', "gid='".$ag['gid']."'");
    // Refresh settings.php
    rebuild_settings();

    // Remove the task entry
    $db->delete_query('tasks', "file='agreement_ip'");
}

// Activate the plugin
function agreement_activate()
{
    global $db, $lang;

    // Template edits
    require_once MYBB_ROOT.'/inc/adminfunctions_templates.php';
    // Remove first to avoid duplicates
    find_replace_templatesets(
        "member_profile",
        "#".preg_quote('{$newterms}')."#i",
        '',
        0
    );
    find_replace_templatesets(
        "usercp",
        "#".preg_quote('{$newterms}')."#i",
        '',
        0
    );
    find_replace_templatesets(
        "newreply",
        "#".preg_quote('{$guestterms}')."#i",
        '',
        0
    );
    find_replace_templatesets(
        "newthread",
        "#".preg_quote('{$guestterms}')."#i",
        '',
        0
    );
    find_replace_templatesets(
        "showthread_quickreply",
        "#".preg_quote('{$guestterms}')."#i",
        '',
        0
    );

    // Now insert the variables
    find_replace_templatesets(
        "member_profile",
        "#".preg_quote('{$memregdate}')."#i",
        ' {$memregdate}{$newterms}'
    );
    find_replace_templatesets(
        "usercp",
        "#".preg_quote('{$regdate}')."#i",
        '{$regdate}{$newterms}'
    );
    find_replace_templatesets(
        "newreply",
        "#".preg_quote('{$attachbox}')."#i",
        '{$attachbox}{$guestterms}'
    );
    find_replace_templatesets(
        "newthread",
        "#".preg_quote('{$attachbox}')."#i",
        '{$attachbox}{$guestterms}'
    );
    find_replace_templatesets(
        "showthread_quickreply",
        "#".preg_quote('</form>')."#i",
        '{$guestterms}</form>'
    );

    // Load language file
    $lang->load('agreement');

    // Check if settings are installed, if not - do it
    $query_ag = $db->simple_select("settings", "*", "name='ag_force'");
    $result = $db->num_rows($query_ag);

    if (!$result) {
        // Add settings
        $query = $db->simple_select("settinggroups", "COUNT(*) as rows");
        $rows = $db->fetch_field($query, "rows");

        // Add Settinggroup
        $agreement_group = array(
            "name" => "UserAgreement",
            "title" => $db->escape_string($lang->ag_group_title),
            "description" => $db->escape_string($lang->ag_group_descr),
            "disporder" => $rows+1,
            "isdefault" => 0
        );
        $db->insert_query("settinggroups", $agreement_group);
        $gid = $db->insert_id();

        // Add settings for settinggroup
        $agreement_1 = array(
            "name" => "ag_force",
            "title" => $db->escape_string($lang->ag_force_title),
            "description" => $db->escape_string($lang->ag_force_descr),
            "optionscode" => "yesno",
            "value" => 1,
            "disporder" => 1,
            "gid" => (int)$gid
            );
        $db->insert_query("settings", $agreement_1);

        $agreement_2 = array(
            "name" => "ag_repeat",
            "title" => $db->escape_string($lang->ag_repeat_title),
            "description" => $db->escape_string($lang->ag_repeat_descr),
            "optionscode" => "yesno",
            "value" => 0,
            "disporder" => 2,
            "gid" => (int)$gid
            );
        $db->insert_query("settings", $agreement_2);

        $agreement_3 = array(
            "name" => "ag_allowedpages",
            "title" => $db->escape_string($lang->ag_allowedpages_title),
            "description" => $db->escape_string($lang->ag_allowedpages_descr),
            "optionscode" => "textarea",
            "value" => '',
            "disporder" => 3,
            "gid" => (int)$gid
            );
        $db->insert_query("settings", $agreement_3);

        $agreement_4 = array(
            "name" => "ag_usercp",
            "title" => $db->escape_string($lang->ag_usercp_title),
            "description" => $db->escape_string($lang->ag_usercp_descr),
            "optionscode" => "yesno",
            "value" => 1,
            "disporder" => 4,
            "gid" => (int)$gid
            );
        $db->insert_query("settings", $agreement_4);

        $agreement_5 = array(
            "name" => "ag_profile",
            "title" => $db->escape_string($lang->ag_profile_title),
            "description" => $db->escape_string($lang->ag_profile_descr),
            "optionscode" => "yesno",
            "value" => 1,
            "disporder" => 5,
            "gid" => (int)$gid
            );
        $db->insert_query("settings", $agreement_5);

        $agreement_6 = array(
            "name" => "ag_lang",
            "title" => $db->escape_string($lang->ag_lang_title),
            "description" => $db->escape_string($lang->ag_lang_descr),
            "optionscode" => "yesno",
            "value" => 0,
            "disporder" => 6,
            "gid" => (int)$gid
            );
        $db->insert_query("settings", $agreement_6);

        $agreement_7 = array(
            "name" => "ag_langtitle",
            "title" => $db->escape_string($lang->ag_langtitle_title),
            "description" => $db->escape_string($lang->ag_langtitle_descr),
            "optionscode" => "text",
            "value" => '',
            "disporder" => 7,
            "gid" => (int)$gid
            );
        $db->insert_query("settings", $agreement_7);
    }

    // Modify the language file field
    $query_lang = $db->simple_select("settings", "*", "name='ag_langtitle'");
    $results_lang = $db->fetch_array($query_lang);

    if (isset($results_lang['optionscode']) && $results_lang['optionscode'] != "text") {
        $update_l = array(
            "optionscode" => "text",
            "value" => ''
            );
        $db->update_query("settings", $update_l, "name='ag_langtitle'");
    }

    // Get the ID of the settings group
    $query_gid = $db->simple_select("settinggroups", "gid", "name='UserAgreement'", array("limit" => 1));
    $gid = $db->fetch_field($query_gid, "gid");

    // Check if ip settings are installed, if not - do it
    $query_ip = $db->simple_select("settings", "*", "name='ag_ipaddresses'");
    $result_ip = $db->num_rows($query_ip);

    if (!$result_ip) {
        $agreement_8 = array(
            "name" => "ag_ipaddresses",
            "title" => $db->escape_string($lang->ag_ipaddresses_title),
            "description" => $db->escape_string($lang->ag_ipaddresses_descr),
            "optionscode" => "select\n
never=".$db->escape_string($lang->ag_never)."\n
day=".$db->escape_string($lang->ag_day)."\n
threedays=".$db->escape_string($lang->ag_threedays)."\n
week=".$db->escape_string($lang->ag_week)."\n
twoweeks=".$db->escape_string($lang->ag_twoweeks)."\n
month=".$db->escape_string($lang->ag_month)."\n
tenweeks=".$db->escape_string($lang->ag_tenweeks)."\n
halfyear=".$db->escape_string($lang->ag_halfyear)."\n
year=".$db->escape_string($lang->ag_year)."\n
forever=".$db->escape_string($lang->ag_forever)."\n",
            "value" => "forever",
            "disporder" => 8,
            "gid" => (int)$gid
            );
        $db->insert_query("settings", $agreement_8);

        $agreement_9 = array(
            "name" => "ag_ip_del_user",
            "title" => $db->escape_string($lang->ag_ip_del_user_title),
            "description" => $db->escape_string($lang->ag_ip_del_user_descr),
            "optionscode" => "yesno",
            "value" => 1,
            "disporder" => 9,
            "gid" => (int)$gid
            );
        $db->insert_query("settings", $agreement_9);
    }

    // Check if guest post settings are installed, if not - do it
    $query_gp = $db->simple_select("settings", "*", "name='ag_guest_post'");
    $result_gp = $db->num_rows($query_gp);

    if (!$result_gp) {
        $agreement_10 = array(
            "name" => "ag_guest_post",
            "title" => $db->escape_string($lang->ag_guest_post_title),
            "description" => $db->escape_string($lang->ag_guest_post_descr),
            "optionscode" => "yesno",
            "value" => 1,
            "disporder" => 10,
            "gid" => (int)$gid
            );
        $db->insert_query("settings", $agreement_10);
    }

    // Refresh settings.php
    rebuild_settings();

    // Have we already added our IP purge task?
    $query = $db->simple_select('tasks', 'tid', "file='agreement_ip'", array('limit' => '1'));
    if ($db->num_rows($query) == 0) {
        // If not then do so
        require_once MYBB_ROOT.'/inc/functions_task.php';

        $ip_task = array(
            "title" => $lang->ag_task_name,
            "file" => 'agreement_ip',
            "description" => $lang->ag_task_description,
            "minute" => 0,
            "hour" => 0,
            "day" => '*',
            "weekday" => '*',
            "month" => '*',
            "nextrun" => TIME_NOW + 3600,
            "lastrun" => 0,
            "enabled" => 1,
            "logging" => 1,
            "locked" => 0,
        );

        $ip_task_id = (int)$db->insert_query('tasks', $ip_task);
        $nextrun = fetch_next_run($ip_task);
        $db->update_query('tasks', "nextrun='{$nextrun}', tid='{$ip_task_id}'");
    }

    // Enable the task
    $db->update_query('tasks', array('enabled' => 1), "file = 'agreement_ip'");
}

// Deactivate the plugin
function agreement_deactivate()
{
    global $db;

    // Remove variables from profile and usercp templates
    require_once MYBB_ROOT.'/inc/adminfunctions_templates.php';
    find_replace_templatesets(
        "member_profile",
        "#".preg_quote('{$newterms}')."#i",
        '',
        0
    );
    find_replace_templatesets(
        "usercp",
        "#".preg_quote('{$newterms}')."#i",
        '',
        0
    );
    find_replace_templatesets(
        "newreply",
        "#".preg_quote('{$guestterms}')."#i",
        '',
        0
    );
    find_replace_templatesets(
        "newthread",
        "#".preg_quote('{$guestterms}')."#i",
        '',
        0
    );
    find_replace_templatesets(
        "showthread_quickreply",
        "#".preg_quote('{$guestterms}')."#i",
        '',
        0
    );

    // Disable the task
    $db->update_query('tasks', array('enabled' => 0), "file = 'agreement_ip'");
}

// Save the agreement
function agreement_save()
{
    global $mybb, $db, $lang;

    // Save the agreement and the timestamp if user has clicked to agree
    if ($mybb->user['uid'] != 0 && !empty($mybb->input['agree']) && $mybb->request_method == "post") {
        $uid = (int)$mybb->user['uid'];
        $updated_record = array(
            "new_terms" => 1,
            "new_terms_date" => TIME_NOW
        );
        $db->update_query("users", $updated_record, "uid='".$uid."'");
        if (!isset($lang->ag_success_accept)) {
            $lang->load('agreement');
        }
        redirect("index.php", $lang->ag_success_accept);
    }
}

// Force existing users to accept the agreement
function agreement_force()
{
    global $mybb, $lang;

    // Forcing members disabled?
    if (isset($mybb->settings['ag_force']) && $mybb->settings['ag_force'] != 1) {
        return;
    }
    // Allow users to log in and out
    if (THIS_SCRIPT == "member.php" && $mybb->input['action'] == "login"
        || THIS_SCRIPT == "member.php" && $mybb->input['action'] == "logout"
    ) {
        return;
    }

    // Get the allowed pages from the setting
    $allowed_pages = explode(PHP_EOL, $mybb->settings['ag_allowedpages']);
    $redir = 0;

    if ($mybb->user['uid'] != 0
        && $mybb->user['new_terms'] != 1
        && $mybb->input['agreement'] != "do"
        && empty($mybb->input['agree'])
    ) {
        if (!isset($lang->ag_force_accept)) {
            $lang->load('agreement');
        }
        // No redirection from allowed pages
        foreach ($allowed_pages as $allowed_page) {
            $allowed_page = trim($allowed_page);
            if (my_strpos($_SERVER['REQUEST_URI'], $allowed_page) !== false) {
                ++$redir;
            }
        }
        if ($redir == 0) {
            redirect(
                "member.php?action=profile&agreement=do",
                $lang->ag_force_accept
            );
        }
    }
}

// Show the agreement
function agreement_show()
{
    global $mybb, $templates, $theme, $header, $footer, $headerinclude, $lang, $memprofile, $newterms;

    if (THIS_SCRIPT == "member.php"
        && $mybb->input['action'] == "profile"
        && $mybb->input['agreement'] == "do"
    ) {
        // Show this page only to users who have not accepted yet or if repeated accepting is allowed or to admins
        if ($mybb->user['uid']
            && ($mybb->user['new_terms'] != 1 || $mybb->settings['ag_repeat'] != 0 || $mybb->usergroup['cancp'] == 1)
        ) {
            eval("\$agreement = \"".$templates->get("member_register_agreement")."\";");
            $agreement = str_replace('value="register"', 'value="profile"', $agreement);
            output_page($agreement);
            exit;
        } elseif (!$mybb->user['uid']) {
            // Show this page to guests, but without the button
            eval("\$agreement = \"".$templates->get("member_register_agreement")."\";");
            $agreement = str_replace($lang->agreement_4, '', $agreement);
            $agreement = str_replace($lang->agreement_5, '', $agreement);
            $agreement = preg_replace('#<input type=\"submit(.*?)\/>#Ui', '', $agreement);
            output_page($agreement);
            exit;
        } else {
            if (!isset($lang->ag_already_accepted)) {
                $lang->load('agreement');
            }
            redirect("index.php", $lang->ag_already_accepted);
        }
    }
}

// Load custom language file
function agreement_lang()
{
    global $mybb, $lang;

    // Load custom language file globally if there is one defined in the settings
    if ($mybb->settings['ag_lang'] != 0 && !empty($mybb->settings['ag_langtitle'])) {
        $langfile = trim($mybb->settings['ag_langtitle']);
        $langfile = htmlspecialchars_uni($mybb->settings['ag_langtitle']);
        if (file_exists('inc/languages/'.$mybb->settings['bblanguage'].'/'.$langfile.'.lang.php')
        ) {
            $lang->load($langfile);
        }
    }
}

// Set agreement and date during registration
function agreement_date(&$user)
{
    $user->user_insert_data['new_terms'] = 1;
    $user->user_insert_data['new_terms_date'] = TIME_NOW;
}

// Display date of accepting agreement in User-CP for the user
function agreement_usercp()
{
    global $mybb, $lang, $newterms;

    if ($mybb->settings['ag_usercp'] != 0 && $mybb->user['new_terms'] != 0) {
        if (!isset($lang->ag_accepted)) {
            $lang->load('agreement');
        }
        $newtermsdate = my_date($mybb->settings['dateformat'], $mybb->user['new_terms_date'], '', 0);
        $newterms = '<br />'.$lang->ag_accepted.$newtermsdate;
    }
}

// Display date of accepting agreement in profile for the user and admins
function agreement_profile()
{
    global $mybb, $memprofile, $lang, $newterms;

    if ($mybb->settings['ag_profile'] != 0
        && $memprofile['new_terms'] != 0
        && ($mybb->usergroup['cancp'] == 1 || $mybb->user['uid'] == $memprofile['uid'])
    ) {
        if (!isset($lang->ag_accepted)) {
            $lang->load('agreement');
        }
        $newtermsdate = my_date($mybb->settings['dateformat'], $memprofile['new_terms_date'], '', 0);
        $newterms = '<br />'.$lang->ag_accepted.$newtermsdate;
    }
}

$plugins->add_hook("admin_config_settings_change", "agreement_settings_change");
// Set peeker in ACP
function agreement_settings_change()
{
    global $db, $mybb, $agreement_settings_peeker;

    $result = $db->simple_select('settinggroups', 'gid', "name='UserAgreement'", array("limit" => 1));
    $group = $db->fetch_array($result);
    $agreement_settings_peeker = ($mybb->input['gid'] == $group['gid']) && ($mybb->request_method != 'post');
}

$plugins->add_hook("admin_settings_print_peekers", "agreement_settings_peek");
// Add peeker in ACP
function agreement_settings_peek(&$peekers)
{
    global $agreement_settings_peeker;

    if ($agreement_settings_peeker) {
        // Peeker for allowed pages settings
        $peekers[] = 'new Peeker($(".setting_ag_repeat"), $("#row_setting_ag_allowedpages"),/0/,true)';
        // Peeker for language file settings
        $peekers[] = 'new Peeker($(".setting_ag_lang"), $("#row_setting_ag_langtitle"),/1/,true)';
    }
}

$plugins->add_hook("admin_tools_action_handler", "agreement_admin_tools_action_handler");
// Set action handler for reset agreements
function agreement_admin_tools_action_handler(&$actions)
{
    $actions['agreement'] = array('active' => 'agreement', 'file' => 'agreement');
}

$plugins->add_hook("admin_load", "agreement_reset");
// Removes the entries of the users agreement to the terms of use
function agreement_reset()
{
    global $mybb, $db, $lang, $page, $run_module, $action_file;

    if ($page->active_action != 'agreement') {
        return false;
    }

    if ($run_module == 'tools' && $action_file == 'agreement') {
        if ($mybb->input['action'] == 'reset') {
            if (!verify_post_check($mybb->get_input('my_post_key'))) {
                flash_message($lang->invalid_post_verify_key2, 'error');
                admin_redirect("index.php?module=config-plugins");
            } else {
                $lang->load('agreement');
                if ($mybb->request_method == "post") {
                    if (empty($mybb->input['no'])) {
                        // Clear accepted terms und date of acceptance
                        $updated_record = array(
                            "new_terms" => 0
                        );
                        $db->update_query('users', $updated_record);

                        log_admin_action(htmlspecialchars_uni($lang->ag_reset));
                        flash_message($lang->ag_reset_success, 'success');
                    }
                    admin_redirect("index.php?module=config-plugins");
                } else {
                    $page->output_confirm_action(
                    "index.php?module=tools-agreement&amp;action=reset",
                    $lang->ag_reset_confirm
                    );
                }
            }
        }
        exit;
    }
}

$plugins->add_hook("datahandler_user_delete_start", "agreement_user_delete");
// Delete all IP's of a user when deleting the user account
function agreement_user_delete(&$dh)
{
    global $mybb, $db;

    if (isset($mybb->settings['ag_ip_del_user']) && $mybb->settings['ag_ip_del_user'] == 1) {
        // Get the uid's of the deleted users
        $delete_uids = implode(',', $dh->delete_uids);

        // Purge all stored IP addresses of the deleted users
        $db->update_query('posts', array('ipaddress' => ''), "uid IN({$delete_uids})");
        $db->update_query('privatemessages', array('ipaddress' => ''), "fromid IN({$delete_uids})");
        $db->update_query('moderatorlog', array('ipaddress' => ''), "uid IN({$delete_uids})");
        $db->update_query('adminlog', array('ipaddress' => ''), "uid IN({$delete_uids})");
        $db->update_query('maillogs', array('ipaddress' => '','fromemail' => ''), "fromuid IN({$delete_uids})");
        $db->update_query('threadratings', array('ipaddress' => ''), "uid IN({$delete_uids})");
        // Delete poll IP's only in MyBB 1.8.15 and later
        if ($mybb->version_code >= 1815) {
            $db->update_query('pollvotes', array('ipaddress' => ''), "uid IN({$delete_uids})");
        }
    }
}

$plugins->add_hook("datahandler_post_insert_post", "agreement_post_ip");
$plugins->add_hook("datahandler_post_insert_thread_post", "agreement_post_ip");
// Don't store IP's for Posts
function agreement_post_ip(&$dh)
{
    global $mybb;

    if (isset($mybb->settings['ag_ipaddresses']) && $mybb->settings['ag_ipaddresses'] == 'never') {
        $dh->post_insert_data['ipaddress'] = '';
    }
}

$plugins->add_hook("datahandler_pm_insert_updatedraft", "agreement_pm_ip");
$plugins->add_hook("datahandler_pm_insert", "agreement_pm_ip");
$plugins->add_hook("datahandler_pm_insert_savedcopy", "agreement_pm_ip");
// Don't store IP's for PM's
function agreement_pm_ip(&$dh)
{
    global $mybb;

    if (isset($mybb->settings['ag_ipaddresses']) && $mybb->settings['ag_ipaddresses'] == 'never') {
        $dh->pm_insert_data['ipaddress'] = '';
    }
}

$plugins->add_hook("polls_vote_process", "agreement_poll_ip");
// Don't store IP's for Poll Votes
function agreement_poll_ip()
{
    global $mybb, $db, $poll;

    if (isset($mybb->settings['ag_ipaddresses']) && $mybb->settings['ag_ipaddresses'] == 'never') {
        $db->update_query("polls", array('ipaddress' => ''), "pid='".$poll['pid']."'");
    }
}

$plugins->add_hook("newreply_start", "agreement_guest_box");
$plugins->add_hook("newthread_start", "agreement_guest_box");
$plugins->add_hook("showthread_start", "agreement_guest_box");
// Display checkbox for guest posts
function agreement_guest_box()
{
    global $mybb, $lang, $guestterms;

    // Function disabled?
    if (isset($mybb->settings['ag_guest_post']) && $mybb->settings['ag_guest_post'] != 1) {
        return;
    }

    $guestterms = '';
    if (!isset($lang->ag_guestterms)) {
        $lang->load('agreement');
    }

    // Display checkbox for guests only
    if (!$mybb->user['uid']) {
        $guestterms = '<br />
<div align="center">
    <label><input type="checkbox" class="checkbox" name="guestterms" value="1" /> '.$lang->ag_guestterms.'</label>
<br /></div>';
    }
}

$plugins->add_hook("datahandler_post_validate_post", "agreement_guest_post");
// Allow guest posts only if he agreed to the terms of use
function agreement_guest_post(&$dh)
{
    global $mybb, $lang;

    // Function disabled?
    if (isset($mybb->settings['ag_guest_post']) && $mybb->settings['ag_guest_post'] != 1) {
        return;
    }

    // Check if we have a guest
    if ($mybb->user['uid'] == 0 && $dh->method == "insert") {
       $terms_agreed  = $mybb->get_input('guestterms', MyBB::INPUT_INT);
        // Box was not checked, display error
        if ($terms_agreed != 1) {
            if (!isset($lang->ag_guestterms_notaccepted)) {
                $lang->load('agreement');
            }
            $dh->set_error($lang->ag_guestterms_notaccepted);
            return false;
        }
        return true;
    }
}

$plugins->add_hook("datahandler_post_validate_thread", "agreement_guest_thread");
// Allow guest threads only if he agreed to the terms of use
function agreement_guest_thread(&$dh)
{
    global $mybb, $lang;

    // Function disabled?
    if (isset($mybb->settings['ag_guest_post']) && $mybb->settings['ag_guest_post'] != 1) {
        return;
    }

    // Check if we have a guest
    if ($mybb->user['uid'] == 0 && $dh->method == "insert") {
       $terms_agreed  = $mybb->get_input('guestterms', MyBB::INPUT_INT);
        // Box was not checked, display error
        if ($terms_agreed != 1) {
            if (!isset($lang->ag_guestterms_notaccepted)) {
                $lang->load('agreement');
            }
            $dh->set_error($lang->ag_guestterms_notaccepted);
            return false;
        }
        return true;
    }
}
